# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-04-28 15:45


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
        ('admin', '0009_auto_20170420_1238'),
    ]

    operations = [
        migrations.CreateModel(
            name='Permission',
            fields=[
            ],
            options={
                'verbose_name': 'Permiss\xe3o',
                'proxy': True,
                'verbose_name_plural': 'Permiss\xf5es',
                'indexes': [],
            },
            bases=('auth.permission',),
        ),
        migrations.AlterModelOptions(
            name='group',
            options={'verbose_name': 'Grupo', 'verbose_name_plural': 'Grupos'},
        ),
    ]



# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-06-19 16:48


from django.db import migrations
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0011_user_permission_mapping'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='organization',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Organization', verbose_name='Organiza\xe7\xe3o'),
        ),
        migrations.AddField(
            model_name='user',
            name='unit',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Unit', verbose_name='Unidade'),
        ),
    ]



# -*- coding: utf-8 -*-

import re
import inspect
from django.apps import apps
from django.utils.translation import ugettext as _
from djangoplus.utils.metadata import get_metadata, get_fiendly_name


def extract_documentation(model):
    s = ''
    if model.__doc__:
        if not model.__doc__.startswith('{}('.format(model.__name__)):
            s = model.__doc__
            s = s.replace('\t', '').replace('\n', '').replace('  ', ' ').strip()
    return s


def extract_exception_messages(function):
    messages = []
    if function:
        code = ''.join([x for x in inspect.getsourcelines(function)[0]])
        for message in re.findall('ValidationError.*\(.*\)', code):
            messages.append(
                message[message.index('(') + 1:message.index(')') - 1].replace('u\'', '').replace('"', ''))
    return messages


def get_search_fields(model):
    l = []
    lookups = get_metadata(model, 'search_fields', [])
    if lookups:
        for i, lookup in enumerate(lookups):
            l.append('"{}"'.format(get_fiendly_name(model, lookup).lower()))
            if i > 0 and i == len(lookups) - 2:
                l.append(_(' or '))
            elif i < len(lookups) - 2:
                l.append(',')
    return ''.join(l)


def get_list_filter(model):
    l = []
    lookups = get_metadata(model, 'list_filter', [])
    if lookups:
        for i, lookup in enumerate(lookups):
            l.append('"{}"'.format(get_fiendly_name(model, lookup).lower()))
            if i > 0 and i == len(lookups) - 2:
                l.append(_(' or '))
            elif i < len(lookups) - 2:
                l.append(', ')
    return ''.join(l)


def get_list_display(model):
    l = []
    lookups = get_metadata(model, 'list_display', [])
    for i, lookup in enumerate(lookups):
        if i > 0:
            if i == len(lookups) - 1:
                l.append(_(' and '))
            else:
                l.append(', ')
        l.append('"{}"'.format(get_fiendly_name(model, lookup).lower()))
    return ''.join(l)


def documentation(as_json=False):

    for app_config in apps.get_app_configs():
        description = app_config.module.__doc__



# -*- coding: utf-8 -*-

from django.core.management.commands import migrate
from djangoplus.admin.management import sync_permissions


class Command(migrate.Command):
    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        sync_permissions()



# -*- coding: utf-8 -*-

from django.conf import settings
from django.template import loader
from django.core.mail import EmailMultiAlternatives


def send_mail(subject, message, to, reply_to=None, actions=()):
    from djangoplus.admin.models import Settings
    url = 'http://{}'.format(settings.HOST_NAME or 'localhost:8000')
    app_settings = Settings.default()
    context = dict()
    context['subject'] = subject
    context['project_url'] = url
    context['project_name'] = app_settings.initials
    context['project_description'] = app_settings.name
    context['project_logo'] = app_settings.logo and '{}/media/{}'.format(url, app_settings.logo) or \
        '{}/static/images/mail.png'.format(url)
    context['actions'] = actions
    context['message'] = message.replace('\n', '<br>').replace('\t', '&nbsp;'*4)
    reply_to = reply_to and [reply_to] or None
    from_email = 'Não-Responder <{}>'.format(settings.SERVER_EMAIL)
    html = loader.render_to_string('mail.html', context)
    email = EmailMultiAlternatives(subject, 'Mensagem em anexo.', from_email, [to], reply_to=reply_to)
    email.attach_alternative(html, "text/html")
    return email.send()





# -*- coding: utf-8 -*-

import uuid
import re
import base64
from datetime import datetime, date
from django import forms
from decimal import Decimal
from django.apps import apps
from django.core import validators
from djangoplus.ui.components.forms import widgets
from django.core.exceptions import ValidationError
from django.core.files.base import ContentFile


# Base Fields #


class CharField(forms.fields.CharField):
    widget = widgets.TextInput


class EmailField(CharField):
    widget = widgets.EmailInput


class UrlField(CharField):
    widget = widgets.UrlInput


class AddressField(CharField):
    widget = widgets.AddressInput


class TypedChoiceField(forms.fields.TypedChoiceField):
    widget = widgets.SelectWidget


class TextField(forms.fields.CharField):
    def __init__(self, *args, **kwargs):
        kwargs.pop('widget', widgets.Textarea)
        super(TextField, self).__init__(*args, widget=widgets.Textarea, **kwargs)


class FormattedTextField(forms.fields.CharField):
    def __init__(self, *args, **kwargs):
        kwargs.pop('widget', widgets.Textarea)
        super(FormattedTextField, self).__init__(*args, widget=widgets.FormattedTextarea, **kwargs)


class IntegerField(forms.fields.IntegerField):
    widget = widgets.NumberInput

    def __init__(self, *args, **kwargs):
        super(IntegerField, self).__init__(*args, **kwargs)
        self.widget.min_value = self.min_value
        self.widget.max_value = self.max_value


class DecimalField(forms.fields.DecimalField):
    widget = widgets.DecimalInput

    def to_python(self, value):
        if not value in validators.EMPTY_VALUES:
            return Decimal(value.replace('.', '').replace(',', '.'))
        return None


class MoneyField(DecimalField):
    widget = widgets.MoneyInput


class DateField(forms.fields.DateField):
    widget = widgets.DateWidget


class CurrentDateTimeField(DateField):
    widget = widgets.HiddenInput

    def clean(self, value):
        return date.today()


class PastDateField(forms.fields.DateField):
    widget = widgets.DateWidget

    def clean(self, value):
        value = super(PastDateField, self).clean(value)
        if value and value > date.today():
            raise ValidationError('Informe uma data atual ou passada')
        return value


class FutureDateField(forms.fields.DateField):
    widget = widgets.DateWidget

    def clean(self, value):
        value = super(FutureDateField, self).clean(value)
        if value and value < date.today():
            raise ValidationError('Informe uma data atual ou futura')
        return value


class DateTimeField(forms.fields.DateTimeField):
    widget = widgets.DateTimeWidget


class CurrentDateTimeField(DateTimeField):
    widget = widgets.HiddenInput

    def clean(self, value):
        return datetime.now()


class PastDateTimeField(forms.fields.DateTimeField):
    widget = widgets.DateTimeWidget

    def clean(self, value):
        value = super(PastDateTimeField, self).clean(value)
        if value and value > datetime.now():
            raise ValidationError('Informe uma data/hora atual ou passada')
        return value


class FutureDateTimeField(forms.fields.DateTimeField):
    widget = widgets.DateTimeWidget

    def clean(self, value):
        value = super(FutureDateTimeField, self).clean(value)
        if value and value < datetime.now():
            raise ValidationError('Informe uma data/hora atual ou futura')
        return value


class BooleanField(forms.fields.BooleanField):
    widget = widgets.CheckboxInput


class NullBooleanField(forms.fields.NullBooleanField):
    widget = widgets.NullBooleanSelect


# Model Choice Field #

class ChoiceField(forms.fields.ChoiceField):
    widget = widgets.SelectWidget


class ModelChoiceField(forms.models.ModelChoiceField):
    widget = widgets.SelectWidget

    def __init__(self, *args, **kwargs):
        minimum_input_length = kwargs.pop('minimum_input_length', 3)
        self.lazy = kwargs.pop('lazy', False)
        self.pick = kwargs.pop('pick', False)
        self.ignore_lookup = kwargs.pop('ignore_lookup', False)
        self.form_filters = kwargs.pop('form_filters', [])
        super(ModelChoiceField, self).__init__(*args, **kwargs)
        if self.pick:
            self.widget = widgets.PickWidget()
        else:
            self.widget.lazy = self.lazy
            self.widget.form_filters = self.form_filters
            self.widget.minimum_input_length = minimum_input_length


class OneToOneField(ModelChoiceField):
    pass


# Multiple Model Choice Fields #

class MultipleModelChoiceField(forms.models.ModelMultipleChoiceField):
    widget = widgets.SelectMultipleWidget

    def __init__(self, *args, **kwargs):
        minimum_input_length = kwargs.pop('minimum_input_length', 3)
        self.lazy = kwargs.pop('lazy', False)
        self.pick = kwargs.pop('pick', False)
        self.ignore_lookup = kwargs.pop('ignore_lookup', False)
        self.form_filters = kwargs.pop('form_filters', [])
        super(MultipleModelChoiceField, self).__init__(*args, **kwargs)
        if self.pick:
            self.widget = widgets.PickWidget(multiple=True)
        else:
            self.widget.lazy = self.lazy
            self.widget.form_filters = self.form_filters
            self.widget.minimum_input_length = minimum_input_length


class OneToManyField(MultipleModelChoiceField):
    pass


# File Fields #


class FileField(forms.fields.FileField):
    widget = widgets.FileInput

    def clean(self, data, initial=None):
        cleaned_data = super(FileField, self).clean(data, initial=initial)
        if cleaned_data and cleaned_data.name != initial:
            extension = cleaned_data.name.split('.')[-1]
            cleaned_data.name = '{}.{}'.format(str(uuid.uuid1()), extension)
        return cleaned_data


class ImageField(forms.fields.ImageField):
    widget = widgets.ImageInput

    def clean(self, data, initial=None):
        cleaned_data = super(ImageField, self).clean(data, initial=initial)
        if cleaned_data and cleaned_data.name != initial:
            extension = cleaned_data.name.split('.')[-1]
            cleaned_data.name = '{}.{}'.format(str(uuid.uuid1()), extension)
        return cleaned_data


class PhotoField(ImageField):
    widget = widgets.PhotoWidget

    def to_python(self, data):
        if data:
            data = ContentFile(base64.b64decode(data.split(',')[1]))
            data.name = 'Image.png'
        return super(PhotoField, self).to_python(data)


# Regional Fields #

class CpfField(CharField):
    widget = widgets.CpfWidget

    default_error_messages = {
        'invalid': 'Número de CPF inválido.',
        'max_digits': 'O CPF deve conter 11 dígitos',
        'digits_only': 'O CPF deve conter apenas dígitos',
    }

    def __init__(self, *args, **kwargs):
        super(CpfField, self).__init__(min_length=11, *args, **kwargs)

    def dv(self, v):
        if v >= 2:
            return 11 - v
        return 0

    def clean(self, value):
        value = super(CpfField, self).clean(value)
        if value in validators.EMPTY_VALUES:
            return ''
        orig_value = value[:]
        if not value.isdigit():
            value = re.sub("[-\.]", "", value)
        try:
            int(value)
        except ValueError:
            raise ValidationError(self.error_messages['digits_only'])
        if len(value) != 11:
            raise ValidationError(self.error_messages['max_digits'])
        orig_dv = value[-2:]

        new_1dv = sum([i * int(value[idx]) for idx, i in enumerate(range(10, 1, -1))])
        new_1dv = self.dv(new_1dv % 11)
        value = value[:-2] + str(new_1dv) + value[-1]
        new_2dv = sum([i * int(value[idx]) for idx, i in enumerate(range(11, 1, -1))])
        new_2dv = self.dv(new_2dv % 11)
        value = value[:-1] + str(new_2dv)
        if value[-2:] != orig_dv:
            raise ValidationError(self.error_messages['invalid'])

        return orig_value


class CnpjField(CharField):
    widget = widgets.CnpjWidget
    default_error_messages = {
        'invalid': 'Número de CNPJ inválido.',
        'max_digits': 'O CNPJ deve conter 18 dígitos',
        'digits_only': 'O CNPJ deve conter apenas dígitos',
    }

    def dv(self, v):
        if v >= 2:
            return 11 - v
        return 0

    def clean(self, value):
        value = super(CnpjField, self).clean(value)
        if value in validators.EMPTY_VALUES:
            return ''
        orig_value = value[:]
        if not value.isdigit():
            value = re.sub("[-/\.]", "", value)
        try:
            int(value)
        except ValueError:
            raise ValidationError(self.error_messages['digits_only'])
        if len(value) != 14:
            raise ValidationError(self.error_messages['max_digits'])
        orig_dv = value[-2:]

        new_1dv = sum([i * int(value[idx]) for idx, i in enumerate(list(range(5, 1, -1)) + list(range(9, 1, -1)))])
        new_1dv = self.dv(new_1dv % 11)
        value = value[:-2] + str(new_1dv) + value[-1]
        new_2dv = sum([i * int(value[idx]) for idx, i in enumerate(list(range(6, 1, -1)) + list(range(9, 1, -1)))])
        new_2dv = self.dv(new_2dv % 11)
        value = value[:-1] + str(new_2dv)
        if value[-2:] != orig_dv:
            raise ValidationError(self.error_messages['invalid'])

        return orig_value


class CpfCnpjField(CharField):
    widget = widgets.CpfCnpjWidget


class CepField(forms.fields.RegexField):
    widget = widgets.CepWidget

    default_error_messages = {
        'invalid': 'O CEP deve estar no formato XX.XXX-XXX.',
    }

    def __init__(self, *args, **kwargs):
        super(CepField, self).__init__(r'^\d{2}\.\d{3}-\d{3}$',
                                       *args, **kwargs)


class CarPlateField(CharField):
    widget = widgets.CarPlateWidget

    def __init__(self, *args, **kwargs):
        super(CarPlateField, self).__init__(min_length=8, *args, **kwargs)


class PhoneField(CharField):
    PHONE_DIGITS_RE = re.compile(r'^(\d{2})[-\.]?(\d{4,5})[-\.]?(\d{4})$')

    widget = widgets.PhoneWidget

    default_error_messages = {
        'invalid': 'O telefone deve estar no formato (XX) XXXX-XXXX.',
    }

    def clean(self, value):
        super(PhoneField, self).clean(value)
        if value in validators.EMPTY_VALUES:
            return ''
        value = re.sub('(\(|\)|\s+)', '', str(value))
        m = PhoneField.PHONE_DIGITS_RE.search(value)
        if m:
            return '({}) {}-{}'.format(m.group(1), m.group(2), m.group(3))
        raise ValidationError(self.error_messages['invalid'])


# Utilitary Fields #

class PasswordField(CharField):
    widget = widgets.PasswordInput


class DecimalField3(CharField):
    widget = widgets.DecimalInput3

    def __init__(self, *args, **kwargs):
        kwargs.pop('max_digits')
        kwargs.pop('decimal_places')
        super(DecimalField3, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if not value in validators.EMPTY_VALUES:
            return Decimal(value.replace('.', '').replace(',', '.'))
        return None


class CurrentUserField(ModelChoiceField):
    widget = widgets.HiddenInput

    def __init__(self, *args, **kwargs):
        User = apps.get_app_config('admin').models['user']
        kwargs['queryset'] = User.objects.all()
        super(CurrentUserField, self).__init__(*args, **kwargs)


class DateRangeField(forms.fields.MultiValueField):
    widget = widgets.DateRangeWidget

    def __init__(self, fields=(DateField(required=False), DateField(required=False)), sum_end_date=False, *args,
                 **kwargs):
        super(forms.fields.MultiValueField, self).__init__(*args, **kwargs)

    def clean(self, value):
        """
        Returns [(data), (data+1 dia)] in order to make  .filter(date__gt=start_date, date__lt=end_date) easier
        """
        datefield = DateField(required=False)
        try:
            start_date, end_date = [datefield.clean(i) for i in value]
        except:
            raise ValidationError('A faixa de datas está inválida.')
        if start_date > end_date:
            raise ValidationError('A data final é menor que a inicial.')
        return [start_date, end_date]


class DateFilterField(DateRangeField):
    widget = widgets.DateFilterWidget

    def __init__(self, *args, **kwargs):
        super(DateFilterField, self).__init__(*args, **kwargs)
        self.widget.label = self.label


class CreditCardField(CharField):
    widget = widgets.CreditCardWidget

    def __init__(self, *args, **kwargs):
        super(CreditCardField, self).__init__(min_length=19, *args, **kwargs)


class OneDigitValidationField(CharField):
    widget = widgets.OneDigitValidationInput


# -*- coding: utf-8 -*-


# list - Allows the user to access the Admin page to view all registered objects
# add - Allows the user to add objects from the Admin page and the Relation tables
# edit - Allows the user to edit objects from the Admin page and the Relation tables
# delete - Allows the user to delete objects from Admin page and Relation tables


def has_permission(request, model, perm):
    app_label = getattr(model, '_meta').app_label
    model_name = model.__name__.lower()
    return request.user.has_perm('{}.{}_{}'.format(app_label, perm, model_name))


def has_list_permission(request, model):
    return has_permission(request, model, 'list')


def has_view_permission(request, model):
    return has_permission(request, model, 'view')


def has_add_permission(request, model):
    return has_permission(request, model, 'add')


def has_relate_permission(request, model):
    return has_permission(request, model, 'relate')


def has_edit_permission(request, model):
    return has_permission(request, model, 'edit')


def has_delete_permission(request, model):
    return has_permission(request, model, 'delete')


def has_view_permission(request, model):
    return has_permission(request, model, 'view')


def can(request, obj, action):
    if request.user.is_superuser:
        return True
    if has_permission(request, type(obj), action):
        func_name = 'can_{}'.format(action)
        if hasattr(obj, func_name):
            obj.request = request
            return getattr(obj, func_name)()
        else:
            return True
    else:
        return False


def can_add(request, obj):
    return can(request, obj, 'add')


def can_edit(request, obj):
    return can(request, obj, 'edit')


def can_delete(request, obj):
    return can(request, obj, 'delete')


def can_view(request, obj):
    return can(request, obj, 'view')


def check_group_or_permission(request, perm_or_group, ignore_superuser=False):
    if perm_or_group:
        satisfied = False
        for perm_or_group in (type(perm_or_group) not in (list, tuple) and (perm_or_group,) or perm_or_group):
            if perm_or_group:
                if request.user.is_superuser and not ignore_superuser:
                    satisfied = True
                elif '.' not in perm_or_group:
                    satisfied = request.user.groups.filter(name=perm_or_group).exists()
                else:
                    satisfied = request.user.has_perm(perm_or_group)
            else:
                satisfied = True
            if satisfied: break
        return satisfied
    return True



# -*- coding: utf-8 -*-

"""
Django settings for mysite project.

Generated by 'django-admin startproject' using Django 1.8.5.

For more information on this file, see
https://docs.djangoproject.com/en/1.8/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.8/ref/settings/
"""

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.8/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'z4_1u0s82j$e$)s)23ha+oqryrgx9#lnp68-#h(##%m4y0k1$t'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

HOST_NAME = 'localhost:8000'

# Application definition

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'djangoplus',
    'djangoplus.docs',
    'djangoplus.test',
    'djangoplus.admin',
    'djangoplus.ui',
    'djangoplus.ui.components.forms',
    'djangoplus.ui.components.calendar',
    'djangoplus.ui.components.select',
    'djangoplus.ui.components.navigation',
    'djangoplus.ui.components.paginator',
    'djangoplus.ui.components.panel',
    'djangoplus.ui.components.utils',
    'djangoplus.ui.components.editor',
)

APP_MAPPING = {
    'admin': 'djangoplus.admin',
    'guide': 'djangoplus.ui.components.guide',
    'docs': 'djangoplus.docs'
}

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'djangoplus.ui.ComponenteResponseMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'djangoplus.admin.context_processors.context_processor'
            ],
            'builtins': [
                'djangoplus.templatetags',
                'djangoplus.ui.components.paginator.templatetags'
            ],
        },
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.8/topics/i18n/

LANGUAGE_CODE = 'pt-br'

TIME_ZONE = 'America/Recife'

USE_I18N = True

USE_L10N = True

USE_TZ = False

STATIC_URL = '/static/'

MEDIA_URL = '/media/'

AUTH_USER_MODEL = 'admin.User'

EXTRA_JS = []

EXTRA_CSS = []

DEFAULT_SUPERUSER = 'admin'
DEFAULT_PASSWORD = 'senha'
USERNAME_MASK = None
DROPBOX_TOKEN = ''
BACKUP_FILES = ['media', 'sqlite.db']

LOGGING_ = {
    'version': 1,
    'filters': {
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        }
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'filters': ['require_debug_true'],
            'class': 'djangoplus.utils.logging.CustomStreamHandler',
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'DEBUG',
            'handlers': ['console'],
        }
    }
}

GRADIENT = ('#aeb6bf', '#85929e', '#5d6d7e', '#d6dbdf', '#34495e', '#2e4053')

CAPTCHA_KEY = '6Lcpc1IUAAAAADXT-pykYw7_duD4NkrQ54VXjlfJ'
CAPTCHA_SECRET = '6Lcpc1IUAAAAABiWOqddzw63lR6ZEaQTtpuM-X5m'

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'



# -*- coding: utf-8 -*-
from logging import StreamHandler


class CustomStreamHandler(StreamHandler):
    def emit(self, record):
        sql = record.getMessage()
        sql = sql.replace('\n            ', '')
        time = sql[0:8]
        if 'SELECT' in sql:
            sql = sql[8:]
            if 'COUNT' not in sql and 'FROM' in sql:
                sql = 'SELECT * {}'.format(sql[sql.index('FROM'):])
        elif 'BEGIN' in sql:
            sql = None
        elif 'UPDATE "django_session"' in sql:
            sql = None

        if sql:
            print(time, sql)



# -*- coding: utf-8 -*-

from importlib import import_module
from django.conf import settings


modules = list()

modules.append(import_module('django.template.defaultfilters'))
for item in settings.TEMPLATES:
    for builtins in item['OPTIONS']['builtins']:
        modules.append(import_module(builtins))


def apply_filter(obj, filter_name, **kwargs):
    for module in modules:
        if hasattr(module, filter_name):
            func = getattr(module, filter_name)
            return func(obj, **kwargs)
    return obj



# -*- coding: utf-8 -*-


from fabric.api import *
from fabric.contrib.files import exists, append, contains
from os import path
import sys
import os
import json
import datetime
import time
import io

username = 'root'
project_dir = os.getcwd()
project_name = project_dir.split('/')[-1]
remote_project_dir = '/var/opt/{}'.format(project_name)

if 'deploy' in sys.argv or 'undeploy' in sys.argv or 'update' in sys.argv or 'push' in sys.argv or 'backupdb' in sys.argv:
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{}.settings'.format(project_name))
    from django.conf import settings

env.user = username
env.connection_attempts = 10

GIT_INGORE_FILE_CONTENT = '''*.pyc
.svn
.DS_Store
.DS_Store?
._*
.idea/*
.Spotlight-V100
.Trashes
ehthumbs.db
Thumbs.db
.project
.pydevproject
.settings/*
sqlite.db
media/*
mail/*
fabfile.pyc
'''
NGINEX_FILE_CONTENT = '''server {{
    client_max_body_size 100M;
    listen {port};
    server_name {server_name};
    access_log /var/opt/{project_name}/logs/nginx_access.log;
    error_log /var/opt/{project_name}/logs/nginx_error.log;
    location /static {{
        alias /var/opt/{project_name}/static;
    }}
    location /media {{
        alias /var/opt/{project_name}/media;
    }}
    location / {{
        proxy_pass_header Server;
        proxy_set_header Host $http_host;
        proxy_redirect off;
        proxy_set_header X-Real_IP $remote_addr;
        proxy_set_header X-Scheme $scheme;
        proxy_connect_timeout 600s;
        proxy_send_timeout 600s;
        proxy_read_timeout 600s;
        proxy_pass http://localhost:{local_port}/;
    }}
}}
'''
SUPERVISOR_FILE_CONTENT = '''[program:{project_name}]
directory = /var/opt/{project_name}
user = www-data
command = /var/opt/{project_name}/gunicorn_start.sh
stdout_logfile = /var/opt/{project_name}/logs/supervisor_out.log
stderr_logfile = /var/opt/{project_name}/logs/supervisor_err.log
'''
GUNICORN_FILE_CONTENT = '''#!/bin/bash
set -e
source /var/opt/.virtualenvs/{project_name}/bin/activate
mkdir -p /var/opt/{project_name}/logs
cd /var/opt/{project_name}
exec gunicorn {project_name}.wsgi:application -w 1 -b 127.0.0.1:{port} --timeout=600 --user=www-data --group=www-data --log-level=_debug --log-file=/var/opt/{project_name}/logs/gunicorn.log 2>>/var/opt/{project_name}/logs/gunicorn.log
'''
LIMITS_FILE_CONTENT = '''
*               soft     nofile           65536
*               hard     nofile           65536
root            soft     nofile           65536
root            hard     nofile           65536
'''
BASHRC_FILE_CONTENT = '''
export WORKON_HOME=/var/opt/.virtualenvs
mkdir -p $WORKON_HOME
source /usr/local/bin/virtualenvwrapper.sh
'''


def _debug(s):
    print('[{}] {}\n'.format(datetime.datetime.now(), s))


def _available_port():
    nginex_dir = '/etc/nginx/sites-enabled'
    port = 8000
    with cd(nginex_dir):
        files = run('ls').split()
        files.remove('default')
        if project_name in files:
            files = [project_name]
        if files:
            command = "grep  localhost {} | grep -o '[0-9]*'".format(' '.join(files))
            ports = run(command).split()
            ports.sort()
            port = ports[-1]
            if project_name not in files:
                port = int(port) + 1
    _debug('Returning port {}!'.format(port))
    return int(port)


def _check_local_keys():
    local_home_dir = local('echo $HOME', capture=True)
    local_ssh_dir = path.join(local_home_dir, '.ssh')
    local_public_key_path = path.join(local_ssh_dir, 'id_rsa.pub')
    if not os.path.exists(local_ssh_dir):
        _debug('Creating dir {}...'.format(local_ssh_dir))
        local('mkdir {}'.format(local_ssh_dir))
        if not path.exists(local_public_key_path):
            local("ssh-keygen -f {}/id_rsa -t rsa -N ''".format(local_ssh_dir))

    key = open(local_public_key_path, 'r').read().strip()
    _debug('Checking if private key was uploaded to digital ocean...')
    url = 'https://api.digitalocean.com/v2/account/keys'
    command = '''curl -X GET -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, url)
    response = local(command, capture=True)
    # print response
    if key not in response:
        _debug('Uploading private key to digital ocean...')
        command = '''curl -X POST -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' -d '{{"name":"{}","public_key":"{}"}}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, 'Default', key, url)
        response = local(command, capture=True)
        # print response


def _check_remote_keys():
    local_home_dir = local('echo $HOME', capture=True)
    local_ssh_dir = path.join(local_home_dir, '.ssh')
    local_public_key_path = path.join(local_ssh_dir, 'id_rsa.pub')
    local_private_key_path = path.join(local_ssh_dir, 'id_rsa')

    remote_home_dir = run('echo $HOME')
    remote_ssh_dir = path.join(remote_home_dir, '.ssh')
    remote_public_key_path = path.join(remote_ssh_dir, 'id_rsa.pub')
    remote_private_key_path = path.join(remote_ssh_dir, 'id_rsa')
    remote_private_known_hosts_path = path.join(remote_ssh_dir, 'known_hosts')
    if not exists(remote_ssh_dir):
        _debug('Creading remote dir {}...'.format(remote_ssh_dir))
        run('mkdir -p {}'.format(remote_ssh_dir))
        _debug('Creating empty file {}...'.format(remote_private_known_hosts_path))
        run('touch {}'.format(remote_private_known_hosts_path))

    with cd(remote_ssh_dir):
        public_key = open(local_public_key_path, 'r').read()
        private_key = open(local_private_key_path, 'r').read()
        _debug('Checking if public key is in file {}...'.format(remote_public_key_path))
        if not contains(remote_public_key_path, public_key):
            _debug('Appending public key in file {}...'.format(remote_public_key_path))
            append(remote_public_key_path, public_key)
        _debug('Checking if private key is in file {}...'.format(remote_private_key_path))
        if not contains(remote_private_key_path, private_key):
            _debug('Appending private key in file {}...'.format(remote_private_key_path))
            append(remote_private_key_path, private_key)
        run('chmod 644 {}'.format(remote_public_key_path))
        run('chmod 600 {}'.format(remote_private_key_path))
        _debug('Checking if {} is in file {}...'.format(env.hosts[0], remote_private_known_hosts_path))
        if not contains(remote_private_known_hosts_path, env.hosts[0]):
            _debug('Appending {} in file {}...'.format(env.hosts[0], remote_private_known_hosts_path))
            run('ssh-keyscan {} >> {}'.format(env.hosts[0], remote_private_known_hosts_path))


def _check_repository():
    with cd('/home'):
        git_dir = '/home/git'
        if not exists(git_dir):
            run('adduser --disabled-password --gecos "" git')
            run('mkdir /home/git/.ssh && chmod 700 /home/git/.ssh')
            run('touch /home/git/.ssh/authorized_keys && chmod 600 /home/git/.ssh/authorized_keys')
            run('cat /root/.ssh/authorized_keys >> /home/git/.ssh/authorized_keys')
            run('chown -R git.git /home/git/.ssh/')
        project_git_dir = '/home/git/{}.git'.format(project_name)
        if not exists(project_git_dir):
            run('mkdir {}'.format(project_git_dir))
            run('cd {} && git init --bare'.format(project_git_dir))
            run('chown -R git.git {}'.format(project_git_dir))
    return 'git@{}:{}.git'.format(env.hosts[0], project_name)


def _setup_local_repository():
    _debug('Checking if local project is a git project...')
    if not path.exists(path.join(project_dir, '.git')):
        with cd(project_dir):
            _debug('Making local project a git project...')
            repository_url = _check_repository()
            local('git init')
            local('git remote add origin "{}"'.format(repository_url))
            local('echo "..." > README.md')
            local('echo "{}" > .gitignore'.format(GIT_INGORE_FILE_CONTENT))
            local('git config --global user.email "user@domain.com"')
            local('git config --global user.name "user"')


def _setup_remote_repository():
    _debug('Checking if the project was cloned in remote server...')
    if not exists(remote_project_dir):
        with cd('/var/opt'):
            _debug('Cloning project in remote server...')
            repository_url = _check_repository()
            run('git clone {} {}'.format(repository_url, project_name))
            run('chown -R www-data.www-data {}'.format(project_name))
    _debug('Updating project in remote server...')
    with cd(remote_project_dir):
        run('git pull origin master')


def _push_local_changes():
    _debug('Checking if project has local changes...')
    now = datetime.datetime.now().strftime("%Y%m%d %H:%M:%S")
    with cd(project_dir):
        if 'nothing to commit' not in local('git status', capture=True):
            _debug('Comminting local changes...')
            files = []
            for file_name in local('ls', capture=True).split():
                if file_name not in GIT_INGORE_FILE_CONTENT or file_name == 'fabfile.py':
                    files.append(file_name)
            files.append('.gitignore')
            for pattern in NGINEX_FILE_CONTENT.split():
                if pattern in files:
                    files.remove(pattern)
            local('git add {}'.format(' '.join(files)))
            local("git commit -m '{}'".format(now))
        _debug('Uploading local changes...')
        local('git push origin master')


def _setup_remote_env():
    _debug('Checking if the virtualenv dir was created in remote server...')
    virtual_env_dir = '/var/opt/.virtualenvs'
    if not exists(virtual_env_dir):
        _debug('Creating dir {}'.format(virtual_env_dir))
        run('mkdir -p {}'.format(virtual_env_dir))
    project_env_dir = path.join(virtual_env_dir, project_name)
    _debug('Checking if virtualenv for the project was created...')
    if not exists(project_env_dir):
        with shell_env(WORKON_HOME=virtual_env_dir):
            _debug('Creating virtual env {}'.format(project_name))
            run('source /usr/local/bin/virtualenvwrapper.sh && mkvirtualenv --python=/usr/bin/python3 {}'.format(project_name))


def _setup_remote_project():
    with cd(remote_project_dir):
        _debug('Checking project requirements..')
        if exists('requirements.txt'):
            virtual_env_dir = '/var/opt/.virtualenvs'
            with shell_env(WORKON_HOME=virtual_env_dir):
                _debug('Installing/Updating project requirements...')
                run('source /usr/local/bin/virtualenvwrapper.sh && workon {} && pip3 install --upgrade pip'.format(project_name))
                run('source /usr/local/bin/virtualenvwrapper.sh && workon {} && pip3 install -U -r requirements.txt'.format(project_name))
        _debug('Checking if necessary dirs (logs, media and static) were created...')
        run('mkdir -p logs')
        run('mkdir -p static')
        run('mkdir -p media')
        _debug('Granting access to www-data...')
        run('chown -R www-data.www-data .')


def _check_domain():
    if settings.DIGITAL_OCEAN_DOMAIN:

        url = 'https://api.digitalocean.com/v2/domains'
        command = '''curl -X GET -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' "{}/{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, url, settings.DIGITAL_OCEAN_DOMAIN)
        _debug('Checking if domain {} was created...'.format(settings.DIGITAL_OCEAN_DOMAIN))
        data = json.loads(local(command, capture=True))
        if data.get('id', None) == 'not_found':
            _debug('Creating domain {}...'.format(settings.DIGITAL_OCEAN_DOMAIN))
            ip_address = env.hosts[0]
            command = '''curl -X POST -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' -d '{{"name":"{}","ip_address":"{}"}}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, settings.DIGITAL_OCEAN_DOMAIN, ip_address, url)
            data = json.loads(local(command, capture=True))

        ip_address = None

        try:
            ip_address = local('dig {} a +short'.format(settings.DIGITAL_OCEAN_DOMAIN), capture=True).strip()
        except Exception as e:
            print(e)

        if ip_address != env.hosts[0]:
            _debug('The domain is not activated yet. The ip {} is going to be used for the deploy.'.format(env.hosts[0]))
            return None

    return settings.DIGITAL_OCEAN_DOMAIN


def _print_remote_url():
    file_path = '/etc/nginx/sites-enabled/{}'.format(project_name)
    local_file_path = '/tmp/nginx.tmp'
    get(file_path, local_file_path)
    file_content = open(local_file_path).read()
    server_name = None
    port = None
    for line in file_content.split('\n'):
        if 'server_name ' in line:
            server_name = line.strip().split()[1].replace(';', '')
        elif 'listen ' in line:
            port = line.strip().split()[1].replace(';', '')
    url = 'http://{}'.format(server_name)
    if int(port) != 80:
        url = '{}:{}'.format(url, port)
    print(('\n\n\nURL: {}\n\n'.format(url)))


def _setup_nginx_file():
    file_path = '/etc/nginx/sites-enabled/{}'.format(project_name)
    _debug('Checking nginx file {}...'.format(file_path))
    checked_domain = _check_domain()
    if exists(file_path):
        local_file_path = '/tmp/nginx.tmp'
        get(file_path, local_file_path)
        file_content = open(local_file_path, 'r').read()
        if checked_domain and checked_domain not in file_content:
            content = []
            for line in file_content.split('\n'):
                if 'server_name ' in line:
                    line = line.replace('server_name', 'server_name {}'.format(checked_domain))
                elif 'listen ' in line:
                    line = '    listen 80;'
                content.append(line)
            file_descriptor = open('/tmp/nginx.tmp', 'w')
            file_descriptor.write('\n'.join(content))
            put(file_descriptor, file_path)
            _debug('Restarting nginx...')
            run('/etc/init.d/nginx restart')
    else:
        _debug('Creating nginx file {}...'.format(file_path))
        local_port = _available_port()
        if checked_domain:
            port = 80
            server_name = checked_domain
        else:
            port = local_port + 1000
            server_name = env.hosts[0]
        text = NGINEX_FILE_CONTENT.format(project_name=project_name, server_name=server_name, port=port, local_port=local_port)
        append(file_path, text)
        _debug('Nginx configured with {}:{}'.format(server_name, port))
        _debug('Restarting nginx...')
        run('/etc/init.d/nginx restart')


def _setup_supervisor_file():
    file_path = '/etc/supervisor/conf.d/{}.conf '.format(project_name)
    _debug('Checking supervisor file {}...'.format(file_path))
    if not exists(file_path):
        _debug('Creating supervisor file {}...'.format(file_path))
        text = SUPERVISOR_FILE_CONTENT.format(project_name=project_name)
        append(file_path, text)
        _debug('Reloading supervisorctl...')
        run('supervisorctl reload')


def _setup_gunicorn_file():
    file_path = '/var/opt/{}/gunicorn_start.sh '.format(project_name)
    _debug('Checking gunicorn file {}...'.format(file_path))
    if not exists(file_path):
        _debug('Creating gunicorn file {}'.format(file_path))
        port = _available_port()
        text = GUNICORN_FILE_CONTENT.format(project_name=project_name, port=port)
        append(file_path, text)
        run('chmod a+x {}'.format(file_path))


def _setup_remote_webserver():
    _setup_nginx_file()
    _setup_supervisor_file()
    _setup_gunicorn_file()


def _reload_remote_application():
    _debug('Updating project in remote server...')
    with cd(remote_project_dir):
        virtual_env_dir = '/var/opt/.virtualenvs'
        with shell_env(WORKON_HOME=virtual_env_dir):
            run('source /usr/local/bin/virtualenvwrapper.sh && workon {} && python manage.py sync'.format(project_name))
            run('chown -R www-data.www-data .')
            run('chmod a+w *.db')
            run('ls -l')
            _debug('Restarting supervisorctl...')
            run('supervisorctl restart {}'.format(project_name))


def _delete_remote_project():
    _debug('Deleting remove project...')
    if exists(remote_project_dir):
        run('rm -r {}'.format(remote_project_dir))


def _delete_remote_env():
    _debug('Deleting remote env...')
    run('source /usr/local/bin/virtualenvwrapper.sh && rmvirtualenv {}'.format(project_name))


def _delete_domain():
    url = 'https://api.digitalocean.com/v2/domains'
    if settings.DIGITAL_OCEAN_DOMAIN:
        _debug('Deleting domain {}...'.format(settings.DIGITAL_OCEAN_DOMAIN))
        command = '''curl -X DELETE -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' "{}/{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, url, settings.DIGITAL_OCEAN_DOMAIN)
        local(command)


def _delete_repository():
    project_git_dir = '/home/git/{}.git'.format(project_name)
    if exists(project_git_dir):
        run('rm -r {}'.format(project_git_dir))


def _delete_local_repository():
    _debug('Deleting local repository...')
    with cd(project_dir):
        local('rm -rf .git')


def _delete_nginx_file():
    _debug('Deleting nginx file...')
    file_path = '/etc/nginx/sites-enabled/{} '.format(project_name)
    if exists(file_path):
        run('rm {}'.format(file_path))


def _delete_supervisor_file():
    _debug('Deleting supervisor file..')
    file_path = '/etc/supervisor/conf.d/{}.conf'.format(project_name)
    if exists(file_path):
        run('rm {}'.format(file_path))


def _reload_remote_webserver():
    _debug('Reloading supervisorctl...')
    run('supervisorctl reload')
    _debug('Reloading nginx...')
    run('/etc/init.d/nginx restart')
    _debug('Starting supervisor...')
    run('service supervisor start')


def _configure_crontab():
    _debug('Configuring crontab...')
    output = run("crontab -l")
    line = '0 * * * * /var/opt/.virtualenvs/{}/bin/python /var/opt/{}/manage.py backup >/tmp/cron.log 2>&1'.format(
    project_name, project_name)
    if line not in output:
        run('crontab -l | { cat; echo "{}"; } | crontab -'.format(line))


def _check_droplet():
    _check_local_keys()

    url = 'https://api.digitalocean.com/v2/droplets/'
    command = '''curl -X GET -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, url)
    _debug('Checking if droplet exists...')
    response = json.loads(local(command, capture=True))

    for droplet in response['droplets']:
        ip_address = droplet['networks']['v4'][0]['ip_address']
        if droplet['name'] == project_name or ip_address == settings.DIGITAL_OCEAN_SERVER:
            _debug('Droplet found with IP {}'.format(ip_address))
            local_home_dir = local('echo $HOME', capture=True)
            local_known_hosts_path = path.join(local_home_dir, '.ssh/known_hosts')
            _debug('Checking if file {} exists...'.format(local_known_hosts_path))
            if not os.path.exists(local_known_hosts_path):
                _debug('Creating empty file {}...'.format(local_known_hosts_path))
                local('touch {}'.format(local_known_hosts_path))
            local_known_hosts_file_content = open(local_known_hosts_path, 'r').read()
            if ip_address not in local_known_hosts_file_content:
                _debug('Registering {} as known host...'.format(ip_address))
                time.sleep(5)
                local('ssh-keyscan -T 15 {} >> {}'.format(ip_address, local_known_hosts_path))
                if settings.DIGITAL_OCEAN_SERVER not in local_known_hosts_file_content:
                    _debug('Registering {} as known host...'.format(settings.DIGITAL_OCEAN_SERVER))
                    local('ssh-keyscan {} >> {}'.format(settings.DIGITAL_OCEAN_SERVER, local_known_hosts_path))
            return ip_address

    _debug('No droplet cound be found for the project')


def _create_droplet():
    _check_local_keys()
    if settings.DIGITAL_OCEAN_TOKEN:
        url = 'https://api.digitalocean.com/v2/account/keys'
        _debug('Getting installed keys at digital ocean...')
        command = '''curl -X GET -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, url)
        response = json.loads(local(command, capture=True))
        # print response
        ssh_keys = []
        for ssh_key in response['ssh_keys']:
            ssh_keys.append(ssh_key['id'])

        _debug('Creating droplet...')
        url = 'https://api.digitalocean.com/v2/droplets/'
        command = '''curl -X POST -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' -d '{{"name":"{}","region":"{}","size":"{}","image":"{}", "ssh_keys":{}}}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, project_name, 'nyc3', '512mb', 'debian-8-x64', ssh_keys, url)
        response = json.loads(local(command, capture=True))
        droplet_id = response['droplet']['id']

        time.sleep(10)

        url = 'https://api.digitalocean.com/v2/droplets/{}/'.format(droplet_id)
        command = '''curl -X GET -H 'Content-Type: application/json' -H 'Authorization: Bearer {}' "{}"'''.format(settings.DIGITAL_OCEAN_TOKEN, url)
        response = json.loads(local(command, capture=True))
        # print response
        ip_address = response['droplet']['networks']['v4'][0]['ip_address']
        _debug('Droplet created with IP {}!'.format(ip_address))
        # _update_settings_file(ip_address)
        _execute_aptget()
        return _check_droplet()

    _debug('Please, set the DIGITAL_OCEAN_TOKEN value in settings.py file')
    sys.exit()


def _execute_aptget():
    with cd('/'):
        run('apt-get update')
        run('apt-get -y install python3 python3-pip build-essential python3-dev git nginx supervisor libncurses5-dev')
        run('apt-get -y install vim')
        run(
            'apt-get -y install libjpeg62-turbo-dev libopenjpeg-dev libfreetype6-dev libtiff5-dev liblcms2-dev libwebp-dev tk8.6-dev libjpeg-dev')
        run('apt-get -y install htop')

        if not contains('/etc/security/limits.conf', '65536'):
            # print LIMITS_FILE_CONTENT
            append('/etc/security/limits.conf', LIMITS_FILE_CONTENT)

        run('pip3 install --upgrade pip')
        run('pip3 install virtualenv virtualenvwrapper')

        if not contains('/root/.bashrc', 'WORKON_HOME'):
            # print BASHRC_FILE_CONTENT
            append('/root/.bashrc', BASHRC_FILE_CONTENT)

        if not exists('/swap.img'):
            run('lsb_release -a')
            run('dd if=/dev/zero of=/swap.img bs=1024k count=2000')
            run('mkswap /swap.img')
            run('swapon /swap.img')
            run('echo "/swap.img    none    swap    sw    0    0" >> /etc/fstab')


def _update_settings_file(ip):
    _debug('Updating settings.py file with {} for DIGITAL_OCEAN_SERVER'.format(ip))
    settings_file_path = path.join(settings.BASE_DIR, '{}/settings.py'.format(project_name))
    content = []
    settings_file = open(settings_file_path)
    lines = settings_file.read().split('\n')
    settings_file.close()
    for line in lines:
        if 'DIGITAL_OCEAN_SERVER' in line:
            line = 'DIGITAL_OCEAN_SERVER = \'{}\''.format(ip)
        content.append(line)
    content_str = '\n'.join(content)
    settings_file = open(settings_file_path, 'w')
    settings_file.write(content_str.encode('utf-8'))
    settings_file.close()


def backupdb():
    local_home_dir = local('echo $HOME', capture=True)
    backup_dir = os.path.join(local_home_dir, 'backup')
    if not os.path.exists(backup_dir):
        local('mkdir -p {}'.format(backup_dir))
    with cd('/var/opt'):
        for entry in run('ls').split():
            file_name = '/var/opt/{}/sqlite.db'.format(entry)
            bakcup_file_name = os.path.join(backup_dir, '{}.db'.format(entry))
            if exists(file_name):
                command = 'scp {}@{}:{} {}'.format(username, env.hosts[0], file_name, bakcup_file_name)
                local(command)


def deploy():
    _check_remote_keys()
    _setup_local_repository()
    _push_local_changes()
    _setup_remote_env()
    _setup_remote_repository()
    _setup_remote_project()
    _setup_remote_webserver()
    _reload_remote_application()
    _print_remote_url()


def update():
    _push_local_changes()
    _setup_remote_repository()
    _setup_remote_project()
    _reload_remote_application()
    _setup_nginx_file()
    _print_remote_url()


def push():
    _push_local_changes()
    _setup_remote_repository()
    _reload_remote_application()
    _print_remote_url()


def undeploy():
    _delete_remote_project()
    _delete_domain()
    _delete_repository()
    _delete_local_repository()
    _delete_nginx_file()
    _delete_supervisor_file()
    _reload_remote_webserver()
    _delete_remote_env()


if 'deploy' in sys.argv:
    if settings.DIGITAL_OCEAN_SERVER:
        env.hosts = [_check_droplet()]
    else:
        env.hosts = [_create_droplet()]
elif 'undeploy' in sys.argv or 'update' in sys.argv or 'push' in sys.argv:
    env.hosts = [_check_droplet()]
else:
    env.hosts = []







# -*- coding: utf-8 -*-
import copy
from djangoplus.utils import permissions
from djangoplus.ui.components import forms
from django.db.models.aggregates import Sum
from djangoplus.utils.tabulardata import tolist
from djangoplus.utils.formatter import normalyze
from djangoplus.ui import RequestComponent, ComponentHasResponseException
from djangoplus.ui.components.navigation.dropdown import ModelDropDown, GroupDropDown
from djangoplus.utils.http import CsvResponse, XlsResponse, ReportResponse, mobile
from djangoplus.utils.metadata import get_metadata, get_field, get_fiendly_name, should_filter_or_display, getattr2


class Paginator(RequestComponent):
    def __init__(self, request, qs, title=None, list_display=None, list_filter=None, search_fields=None,
                 list_per_page=25, list_subsets=None, exclude=None, relation=None, readonly=False, is_list_view=False,
                 help_text=None, url=None, template='datagrid.html'):

        super(Paginator, self).__init__(is_list_view and '_' or abs(hash(title)), request)

        self.qs = qs.all()
        self.title = title
        self.list_display = list_display
        self.list_filter = list_filter
        self.search_fields = search_fields
        self.list_per_page = list_per_page
        self.list_subsets = list_subsets
        self.exclude = exclude
        self.relation = relation
        self.readonly = readonly
        self.is_list_view = is_list_view
        self.icon = get_metadata(qs.model, 'icon', None)
        self.list_total = get_metadata(qs.model, 'list_total', None)
        self.ordering = get_metadata(qs.model, 'ordering', None)
        self.template = get_metadata(self.qs.model, 'list_template', template)
        self.help_text = help_text
        self.url = url

        self.subset_actions = []
        self.filters = []
        self.pagination = ''

        self.original_qs = qs
        self.count = None

        # tabs
        self.tabs = []
        self.current_tab = self._get_from_request('tab', None)
        self._load_tabs()

        # list display
        self._configure_list_display()

        # column names
        self.column_names = []
        self._configure_column_names()

        # drop down
        self.paginator_dropdown = GroupDropDown(request)
        self.subset_dropdown = GroupDropDown(request)
        self.drop_down = ModelDropDown(request, qs.model)
        self.mobile = mobile(self.request)

        if hasattr(self.qs, 'permission_map'):
            self.permission_map = self.qs.permission_map
        self.qs = self._filter_queryset(self.qs)

    def get_current_tab_name(self):
        if len(self.tabs) > 1:
            if self.current_tab and self.current_tab != 'all':
                return self.current_tab
        if len(self.tabs) > 0:
            return self.tabs[0][0]
        return ''
    
    def get_tab(self):
        return self._get_from_request('tab', '')
    
    def get_search_fields(self):
        if self.search_fields is not None:
            return self.search_fields
        return get_metadata(self.qs.model, 'search_fields', [])
    
    def get_list_filter(self):
        if self.list_filter is None:
            self.list_filter = copy.deepcopy(get_metadata(self.qs.model, 'list_filter', None))
            if self.list_filter is None:
                self.list_filter = []
                for field in get_metadata(self.qs.model, 'fields'):
                    if field.remote_field and field.remote_field.model:
                        if not field.name.endswith('_ptr') and field.name != self.rel:
                            self.list_filter.append(field.name)
                    elif hasattr(field, 'choices') and field.choices:
                        self.list_filter.append(field.name)
                    elif type(field).__name__ in ['BooleanField', 'NullBooleanField', 'DateField', 'DateTimeField']:
                        self.list_filter.append(field.name)
        return self.list_filter

    def get_filter_form(self):
        form = forms.Form(self.request)
        form.title = 'Filtros'
        form.icon = 'fa-filter'
        form.partial = True
        self.filters = []
        for list_filter in self.get_list_filter():
            if type(list_filter) in (tuple, list):
                field_names = list_filter
            else:
                field_names = list_filter,
            for field_name in field_names:
                if self.relation and field_name == self.relation.hidden_field_name:
                    continue
                field = get_field(self.qs.model, field_name)
                form_field_name = '{}{}'.format(field_name, self.id)
                if hasattr(field, 'auto_now'):
                    initial = (self._get_from_request(field_name, None, '_0'), self._get_from_request(field_name, None, '_1'))
                    form.fields[form_field_name] = forms.DateFilterField(label=normalyze(field.verbose_name), initial=initial, required=False)
                else:
                    initial = self._get_from_request(field_name)
                    if type(field).__name__ in ['BooleanField', 'NullBooleanField']:
                        form.fields[form_field_name] = forms.ChoiceField(
                            choices=[['', ''], ['sim', 'Sim'], ['nao', 'Não'], ], label=normalyze(field.verbose_name), initial=initial, required=False)
                    elif hasattr(field, 'choices') and field.choices:
                        form.fields[form_field_name] = forms.ChoiceField(choices=[['', '']] + field.choices, label=normalyze(field.verbose_name), initial=initial, required=False)
                    else:
                        if hasattr(field.remote_field.model, 'unit_ptr_id') and self.request.user.role_set.filter(scope__unit__isnull=False).values_list('scope__unit', flat=True).count() == 1:
                            continue
                        if hasattr(field.remote_field.model, 'organization_ptr_id') and self.request.user.role_set.filter(scope__organization__isnull=False).values_list('scope__organization', flat=True).count() == 1:
                            continue
                        if field.remote_field and not should_filter_or_display(self.request, self.qs.model, field.remote_field.model):
                            continue
                        if initial:
                            if self.original_qs.query.can_filter():
                                pks = self.original_qs.order_by(field_name).values_list(field_name, flat=True).distinct()
                            else:
                                pks = self.original_qs.model.objects.all().order_by(field_name).values_list(field_name, flat=True).distinct()
                            qs = field.remote_field.model.objects.get_queryset().filter(pk__in=pks)
                            qs = qs | field.remote_field.model.objects.get_queryset().filter(pk=initial)
                        else:
                            if self.qs.query.can_filter():
                                pks = self.qs.order_by(field_name).values_list(field_name, flat=True).distinct()
                            else:
                                pks = self.qs.model.objects.all().order_by(field_name).values_list(field_name, flat=True).distinct()
                            qs = field.remote_field.model.objects.get_queryset().filter(pk__in=pks)
                        empty_label = ''

                        form.fields[form_field_name] = forms.ModelChoiceField(qs, label=normalyze(field.verbose_name), initial=initial, empty_label=empty_label, required=False, lazy=True, ignore_lookup=True, minimum_input_length=0)
                    form.fields[form_field_name].widget.attrs['data-placeholder'] = field.verbose_name
                    if initial:
                        label = form.fields[form_field_name].label
                        value = form.fields[form_field_name].clean(initial)
                        if type(form.fields[form_field_name]) == forms.ChoiceField:
                            for x, y in form.fields[form_field_name].choices:
                                if str(x) == str(value):
                                    value = y
                                    break
                        self.filters.append((label, value))
        return form

    def get_selected_ids(self):
        ids = self.request.GET.get('ids', None)
        if ids:
            ids = ids.split(',')
            ids = 0 not in ids and ids or None
        return ids

    def add_action(self, label, url, css, icon=None):
        self.paginator_dropdown.add_action(label, url, 'global-action {}'.format(css), icon, label)

    def add_subset_action(self, label, url, css, icon=None, subset=None):
        if not subset or self.get_tab() == subset:
            if not self.mobile:
                self.subset_dropdown.add_action(label, url, 'subset-action disabled {}'.format(css), icon, label)

    def add_actions(self):
        export_url = self.request.get_full_path()
        list_csv = get_metadata(self.qs.model, 'list_csv')
        list_xls = get_metadata(self.qs.model, 'list_xls')
        log = get_metadata(self.qs.model, 'log')
        app_label = get_metadata(self.qs.model, 'app_label')
        list_pdf = get_metadata(self.qs.model, 'list_pdf')

        if list_csv:
            export_url = '?' in export_url and '{}&export=csv'.format(export_url) or '{}?export=csv'.format(export_url)
            self.add_action('Exportar CSV', export_url, 'ajax', 'fa-table')

        if list_xls:
            export_url = '?' in export_url and '{}&export=excel'.format(export_url) or '{}?export=excel'.format(export_url)
            self.add_action('Exportar Excel', export_url, 'ajax', 'fa-file-excel-o')

        if log:
            log_url = '/log/{}/{}/'.format(app_label, self.qs.model.__name__.lower())
            if self.request.user.has_perm('admin.list_log'):
                self.add_action('Visualizar Log', log_url, 'ajax', 'fa-history')

        if list_pdf:
            pdf_url = '?' in export_url and '{}&export=pdf'.format(export_url) or '{}?export=pdf'.format(export_url)
            self.add_action('Imprimir', pdf_url, 'ajax', 'fa-print')

        subclasses = self.qs.model.__subclasses__()

        if not subclasses and permissions.has_add_permission(self.request, self.qs.model):
            instance = self.qs.model()
            instance.user = self.request.user
            if not hasattr(instance, 'can_add') or instance.can_add():
                add_label = get_metadata(self.qs.model, 'add_label', 'Cadastrar')
                self.add_action(add_label, '/add/{}/{}/'.format(app_label, self.qs.model.__name__.lower()), 'ajax', 'fa-plus')

        for subclass in subclasses:
            app = get_metadata(subclass, 'app_label')
            verbose_name = get_metadata(subclass, 'verbose_name')
            cls = subclass.__name__.lower()
            if permissions.has_add_permission(self.request, subclass):
                self.add_action(verbose_name, '/add/{}/{}/'.format(app, cls), False, 'fa-plus')

    def get_total(self):
        if self.list_total:
            return self.qs.aggregate(sum=Sum(self.list_total)).get('sum')
        else:
            return None

    def can_show_actions(self):
        return permissions.has_view_permission(self.request, self.qs.model)

    def get_queryset(self, paginate=True):
        queryset = self.qs
        self.count = queryset.count()
        if self.ordering:
            if not type (self.ordering) == tuple:
                self.ordering = self.ordering,
            queryset = queryset.order_by(*self.ordering)

        if paginate:
            l = []

            list_per_page = self._get_list_per_page()
            page_numer = int(self.count / list_per_page + (((self.count % list_per_page) > 0 or self.count < list_per_page) and 1 or 0))
            current_page = int(self._get_from_request('page', 1))
            start = current_page * list_per_page - list_per_page
            end = start + list_per_page
            queryset = queryset[start:end]
            if page_numer > 1:
                l.append(
                    '<ul class="pagination pagination-xs m-top-none pull-right pagination-split">'
                    '<li class="disabled"><a href="#!"><i class="fa fa-chevron-left"></i></a></li>')
                for i in range(current_page-10, current_page+10):
                    if page_numer >= i > 0 and i:
                        onclick = "$('#page{}').val({});$('#{}').submit();".format(self.id, i, self.id)
                        if i == current_page:
                            css = 'active'
                        else:
                            css = 'waves-effect'
                        l.append('<li class="{}"><a href="javascript:" onclick="{}">{}</a></li>'.format(css, onclick, i))
                l.append('<li class="disabled"><a href="#!"><i class="fa fa-chevron-right"></i></a></li></ul>')
            self.pagination = ''.join(l)

        select_related = get_metadata(queryset.model, 'select_related')
        if select_related:
            queryset = queryset.select_related(*select_related)
        return queryset
    
    def get_q(self):
        return self._get_from_request('q', '')

    def _configure_list_display(self):
        hidden_fields = []
        if not self.list_display:
            self.list_display = list(get_metadata(self.qs.model, 'list_display', []))
        if not self.list_display:
            fields = []
            for field in get_metadata(self.qs.model, 'fields'):
                if not hasattr(field, 'display') or field.display:
                    fields.append(field)
            for field in get_metadata(self.qs.model, 'local_many_to_many'):
                if not hasattr(field, 'display') or field.display:
                    fields.append(field)
            for field in fields[1:6]:
                if not field.name.endswith('_ptr') and not field.name == 'ascii' and not type(field).__name__ == 'TreeIndexField':
                    if not hasattr(field, 'display') or field.display:
                        self.list_display.append(field.name)

        for field_name in self.list_display:
            if '__' in field_name:
                attr = getattr2(self.qs.model, field_name)
            else:
                attr = getattr(self.qs.model, field_name)
            if hasattr(attr, 'field_name'):
                field = getattr(self.qs.model, '_meta').get_field(attr.field_name)
                if hasattr(field, 'display') and not field.display:
                    hidden_fields.append(field_name)

            if self.relation and field_name == self.relation.hidden_field_name:
                hidden_fields.append(field_name)

        if self.exclude:
            for field_name in self.exclude:
                if field_name in self.list_display:
                    hidden_fields.append(field_name)

        for field_name in hidden_fields:
            self.list_display.remove(field_name)

    def _configure_column_names(self):
        for lookup in self.list_display:
            hide_field = False
            attr = getattr(self.qs.model, lookup.split('__')[0])
            if hasattr(attr, 'field') and attr.field.remote_field and attr.field.remote_field.model:
                if hasattr(attr.field.remote_field.model, 'unit_ptr_id') and self.request.user.role_set.filter(scope__unit__isnull=False).values_list('scope__unit', flat=True).count() == 1:
                    continue
                if hasattr(attr.field.remote_field.model, 'organization_ptr_id') and self.request.user.role_set.filter(scope__organization__isnull=False).values_list('scope__organization', flat=True).count() == 1:
                    continue
                if not should_filter_or_display(self.request, self.qs.model, attr.field.remote_field.model):
                    hide_field = True
            if not hide_field:
                self.column_names.append(get_fiendly_name(self.qs.model, lookup, as_tuple=True))

    def _load_tabs(self):
        from djangoplus.cache import loader
        if self.list_subsets is None:
            list_subsets = loader.subsets[self.qs.model]
        else:
            list_subsets = []
            for subset_name in self.list_subsets:
                for subset in loader.subsets[self.qs.model]:
                    if subset['name'] == subset_name:
                        list_subsets.append(subset)
        create_default_tab = True
        for subset in list_subsets:
            tab_title = subset['title']
            tab_function = subset['function']
            tab_can_view = subset['can_view']
            tab_name = subset['name']
            tab_help_text = subset['help_text']
            tab_order = subset['order']
            tab_list_display = subset['list_display']
            tab_list_filter = subset['list_filter']
            tab_search_fields = subset['search_fields']
            tab_active = False
            if permissions.check_group_or_permission(self.request, tab_can_view):
                tab_qs = getattr(tab_function.__self__.all(self.request.user), tab_function.__func__.__name__)()
                tab_active = self.current_tab == tab_name
                self.tabs.append([tab_name, tab_title, tab_qs, tab_active, tab_order, tab_help_text])
                if (tab_active or len(list_subsets) == 1) and tab_help_text:
                    self.help_text = tab_help_text
            if tab_name == 'all':
                create_default_tab = False
            if tab_active:
                self.qs = tab_qs
                self.drop_down = ModelDropDown(self.request, self.qs.model)
                self.list_display = tab_list_display
                self.list_filter = tab_list_filter
                self.search_fields = tab_search_fields

        self.tabs = sorted(self.tabs, key=lambda k: k[4])
        if self.list_subsets is None and create_default_tab:
            tab_title = get_metadata(self.qs.model, 'verbose_female') and 'Todas' or 'Todos'
            tab_qs = self.original_qs
            tab_active = self.current_tab == 'all'
            tab_order = 0
            self.tabs.insert(0, ['', tab_title, tab_qs, tab_active, tab_order, None])
        if not self.current_tab and self.tabs:
            self.tabs[0][3] = True
            self.qs = self.tabs[0][2]

    def _get_from_request(self, param_name, default='', suffix=''):
        key = '{}{}{}'.format(param_name, self.id, suffix)
        value = self.request.GET.get(str(self.id)) and self.request.GET.get(key) or default
        return value

    def _get_order_by(self):
        return self._get_from_request('order_by', '')

    def _get_page(self):
        return self._get_from_request('page', 1)

    def _get_list_per_page(self):
        return get_metadata(self.qs.model, 'list_per_page', self.list_per_page)

    def _filter_queryset(self, qs):
        distinct = False
        queryset = None
        search_fields = self.get_search_fields()
        q = self.get_q()
        if q:
            for i, search_field in enumerate(search_fields):
                if i == 0:
                    queryset = qs.filter(**{'{}__icontains'.format(search_field): q})
                else:
                    queryset = queryset | qs.filter(**{'{}__icontains'.format(search_field): q})
        else:
            queryset = qs
        for field_name in self.get_list_filter():
            field = get_field(queryset.model, field_name)
            if type(field).__name__ == 'DateField':
                filter_value = self._get_from_request(field_name, None, '_0')
                if filter_value:
                    date, month, year = filter_value.split('/')
                    filter_value = '{}-{}-{}'.format(year, month, date)
                    queryset = queryset.filter(**{'{}__gte'.format(field_name): filter_value})
                filter_value = self._get_from_request(field_name, None, '_1')
                if filter_value:
                    date, month, year = filter_value.split('/')
                    filter_value = '{}-{}-{}'.format(year, month, date)
                    queryset = queryset.filter(**{'{}__lte'.format(field_name): filter_value})
            elif type(field).__name__ in ['BooleanField', 'NullBooleanField']:
                filter_value = self._get_from_request(field_name)
                if filter_value:
                    filter_value = filter_value == 'sim'
                    queryset = queryset.filter(**{field_name: filter_value})
            else:
                filter_value = self._get_from_request(field_name)
                if filter_value:
                    queryset = queryset.filter(**{field_name: filter_value})
                if type(field).__name__ == 'ManyToManyField':
                    distinct = True
        order_by = self._get_from_request('order_by')

        tree_index_field = hasattr(self.qs.model, 'get_tree_index_field') and self.qs.model().get_tree_index_field() or None
        if tree_index_field:
            queryset = queryset.order_by(tree_index_field.name)
        else:
            if order_by:
                queryset = queryset.order_by(order_by.replace('0', ''))
            else:
                order_by = get_metadata(self.qs.model, 'order_by', iterable=True)
                try:
                    queryset = queryset.order_by(*order_by)
                except AssertionError:
                    # if the querycet was sliced
                    pass
        if distinct:
            queryset = queryset.distinct()
        return queryset

    def process_request(self):
        export = self.request.GET.get('export', None)
        if export == 'pdf':
            raise ComponentHasResponseException(self._get_pdf_response())
        if export == 'csv':
            raise ComponentHasResponseException(self._get_csv_response(self.get_selected_ids()))
        elif export == 'excel':
            raise ComponentHasResponseException(self._get_xls_response(self.get_selected_ids()))

    def _get_csv_response(self, ids=()):
        list_csv = get_metadata(self.qs.model, 'list_csv')
        qs = self.get_queryset(False)
        if ids:
            qs = qs.filter(id__in=ids)
        return CsvResponse(tolist(qs, list_display=list_csv))

    def _get_xls_response(self, ids=()):
        list_xls = get_metadata(self.qs.model, 'list_xls')
        qs = self.get_queryset(False)
        if ids:
            qs = qs.filter(id__in=ids)
        return XlsResponse([('Dados', tolist(qs, list_display=list_xls))])

    def _get_pdf_response(self):
        self.as_pdf = True
        landscape = len(self.list_display) > 4
        return ReportResponse(self.title, self.request, [self], landscape)


# -*- coding: utf-8 -*-

import datetime
from djangoplus.ui.components import forms
from django.contrib import auth
from django.conf import settings
from djangoplus.cache import loader
from djangoplus.ui.components.navigation.breadcrumbs import httprr
from djangoplus.utils.aescipher import encrypt
from djangoplus.admin.models import Group, User, Unit, Settings, Organization
from django.contrib.auth.hashers import make_password


class LoginForm(forms.Form):
    
    ORGANIZATION = 'organization'
    UNIT = 'unit'
    
    username = forms.CharField(label='Usuário')
    password = forms.PasswordField(label='Senha',
                                   help_text='Caso tenha esquecido sua senha, clique aqui para '
                                   '<a class="popup" href="/admin/reset_password/">alterá-la</a>.')

    def __init__(self, request, scope=None, organization=None, unit=None, *args, **kwargs):
        super(LoginForm, self).__init__(request, *args, **kwargs)
        self.scope = None
        self.fields['username'].widget.attrs['class'] = 'input-sm bounceIn animation-delay2'
        if scope:
            if scope == LoginForm.ORGANIZATION or scope == (loader.organization_model and loader.organization_model.__name__.lower()):
                if organization:
                    self.scope_display = organization
                else:
                    organizations = Organization.objects.all()
                    self.fields['login_scope'] = forms.ModelChoiceField(organizations)
                self.scope = LoginForm.ORGANIZATION
            elif scope == LoginForm.UNIT or scope == (loader.unit_model and loader.unit_model.__name__.lower()):
                if unit:
                    self.scope_display = unit
                else:
                    if organization:
                        self.scope_display = organization
                        units = organization.get_units()
                    else:
                        units = Unit.objects.all()
                    self.fields['login_scope'] = forms.ModelChoiceField(units)
                self.scope = LoginForm.UNIT
        self.organization = organization
        self.unit = unit

    def clean(self):
        cleaned_data = super(LoginForm, self).clean()
        username = cleaned_data.get('username')
        password = cleaned_data.get('password')
        if not User.objects.filter(username=username).exists():
            raise forms.ValidationError('Usuário não cadastrado.')
        user = auth.authenticate(username=username, password=password)
        if user is not None:
            if user.active:
                user.permission_mapping = ''
                user.organization = None
                user.unit = None
                if self.scope == LoginForm.ORGANIZATION:
                    user.organization = cleaned_data.get('login_scope', self.organization)
                    is_organization_user = user.role_set.filter(organizations__in=(user.organization, 0)).exists()
                    if not is_organization_user:
                        raise forms.ValidationError('{} não é usuário de {}'.format(username, user.organization))
                elif self.scope == LoginForm.UNIT:
                    user.unit = cleaned_data.get('login_scope', self.unit)
                    is_unit_user = user.role_set.filter(units__in=(user.unit, 0)).exists()
                    is_organization_user = loader.organization_model and user.role_set.filter(organizations__in=(user.unit.get_organization(), 0)).exists()
                    if not is_unit_user and not is_organization_user:
                        raise forms.ValidationError('{} não é usuário de {}'.format(username, user.unit))
                else:
                    roles = user.role_set.all()
                    if roles.count() == 1:
                        role = roles.first()
                        if role.scope:
                            self.request.session['scope'] = str(role.scope)
                            self.request.session.save()
                user.save()
                auth.login(self.request, user)
                return cleaned_data
            else:
                raise forms.ValidationError('Usuário inativo.')
        else:
            raise forms.ValidationError('Senha não confere.')

    def submit(self):
        url = self.request.GET.get('next', '/admin/')
        return httprr(self.request, url, 'Usuário autenticado com sucesso.')


class GroupForm(forms.ModelForm):
    name = forms.CharField(label='Name')

    class Meta:
        title = 'Cadastro de Grupo'
        submit_label = 'Cadastrar'
        icon = 'fa-users'
        model = Group
        fields = ('name',)


class UserForm(forms.ModelForm):
    is_superuser = forms.BooleanField(label='Superusuário', required=False)
    new_password = forms.PasswordField(label='Senha', required=False)

    class Meta:
        model = User
        fields = ('username', 'name', 'active', 'email', 'is_superuser', 'photo')
        title = 'Cadastro de Usuário'
        submit_label = 'Cadastrar'
        icon = 'fa-user'

    def __init__(self, *args, **kwargs):
        super(UserForm, self).__init__(*args, **kwargs)
        if not args[0].user.is_superuser:
            del (self.fields['is_superuser'])
            self.fieldsets = (
                ('Identificação', {'fields': (('name', 'email',), 'photo')}),
                ('Acesso', {'fields': (('username', 'new_password'), ('active',))}),
            )
        else:
            self.fieldsets = (
                ('Identificação', {'fields': (('name', 'email',), 'photo')}),
                ('Acesso', {'fields': (('username', 'new_password'), ('active', 'is_superuser'))}),
            )

    def save(self, *args, **kwargs):
        user = super(UserForm, self).save(commit=False)
        if 'new_password' in self.cleaned_data and self.cleaned_data['new_password']:
            user.set_password(self.cleaned_data['new_password'])
        user.save()


class RegisterForm(forms.Form):
    name = forms.CharField(label='Nome', required=True)
    email = forms.EmailField(label='E-mail', required=True)
    new_password = forms.PasswordField(label='Senha', required=True)
    confirm_password = forms.PasswordField(label='Senha', required=True,
                                           help_text='Repita a senha digitada anteriormente')

    class Meta:
        title = 'Novo Usuário'
        icon = 'fa-user'
        submit_label = 'Cadastrar'
        captcha = True

    fieldsets = (
        ('Dados Gerais', {'fields': ('name', 'email')}),
        ('Senha de Acesso', {'fields': (('new_password', 'confirm_password'))}),
    )

    def clean(self):
        if not self.data['new_password'] == self.data['confirm_password']:
            raise forms.ValidationError('As senhas devem ser iguais.')
        return self.cleaned_data

    def save(self, *args, **kwargs):
        user = User()
        password = self.cleaned_data['new_password']
        user.name = self.cleaned_data['name']
        user.email = self.cleaned_data['email']
        user.username = self.cleaned_data['email']
        user.active = True
        user.is_superuser = False
        user.last_login = None
        user.date_joined = datetime.datetime.now()
        user.username = user.email
        user.set_password(password)
        if True:
            user.save()
            return user
        else:
            token = encrypt('{};{};{};{}'.format(user.first_name, user.last_name, user.email, password))
            url = '{}/admin/create_user/{}/'.format(settings.SERVER_ADDRESS, token)
            user.email_user('Criação de Conta', 'Clique para confirmar a criação de sua conta: {}'.format(url),
                            settings.SYSTEM_EMAIL_ADDRESS)
            return None


class ChangePasswordForm(forms.ModelForm):
    new_password = forms.PasswordField(label='Senha', required=True)
    confirm_password = forms.PasswordField(label='Confirmação', required=True,
                                           help_text='Repita a senha digitada anteriormente')

    fieldsets = (('Senhas', {'fields': ('new_password', 'confirm_password',)}),)

    class Meta:
        model = User
        fields = ('id',)
        title = 'Alteração de Senha'
        submit_label = 'Alterar'
        icon = 'fa-key'

    def clean(self):
        if not self.data.get('new_password') == self.data.get('confirm_password'):
            raise forms.ValidationError('As senhas devem ser iguais.')
        return self.cleaned_data

    def save(self, *args, **kwargs):
        super(ChangePasswordForm, self).save(*args, **kwargs)
        User.objects.filter(pk=self.instance.pk).update(password=make_password(self.cleaned_data['new_password']))
        user = auth.authenticate(username=self.instance.username, password=self.cleaned_data['new_password'])
        auth.login(self.request, user)


class RecoverPassowordForm(forms.Form):
    email = forms.EmailField(label='E-mail')

    class Meta:
        captcha = True

    def clean(self):
        cleaned_data = super(RecoverPassowordForm, self).clean()
        qs = User.objects.filter(
            email=cleaned_data['email']
        )
        if qs.exists():
            qs[0].send_access_invitation()
            return cleaned_data
        else:
            raise forms.ValidationError('Usuário não encontrado')


class ProfileForm(forms.ModelForm):
    new_password = forms.PasswordField(label='Senha', required=False)
    confirm_password = forms.PasswordField(label='Confirmação', required=False,
                                           help_text='Repita a senha digitada anteriormente')

    class Meta:
        model = User
        fields = ('name', 'photo', 'username', 'email')
        title = 'Atualização de Perfil'
        submit_label = 'Atualizar Perfil'
        icon = 'fa-user'

    fieldsets = (
        ('Dados do Usuário', {'fields': (('name', 'username'), 'email', 'photo')}),
        ('Senha de Acesso', {'fields': (('new_password', 'confirm_password'),)}),
    )

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)
        self.username = self.instance.username

    def clean_confirm_password(self):
        if not self.data.get('new_password') == self.data.get('confirm_password'):
            raise forms.ValidationError('As senhas devem ser iguais.')
        return self.cleaned_data

    def save(self, *args, **kwargs):
        super(ProfileForm, self).save(*args, **kwargs)
        password = self.cleaned_data['new_password']

        if password:
            self.instance.set_password(password)
            self.instance.save()
            user = auth.authenticate(username=self.cleaned_data.get('username'), password=password)
        else:
            user = self.instance
        user.backend = self.request.session[auth.BACKEND_SESSION_KEY]
        auth.login(self.request, user)

        if self.username != user.username:
            for model in loader.role_models:
                attr = loader.role_models[model]['username_field']
                model.objects.filter(**{attr: self.username}).update(**{attr: self.request.user.username})


class SettingsForm(forms.ModelForm):
    class Meta():
        model = Settings
        exclude = ()

    def __init__(self, *args, **kwargs):
        kwargs['instance'] = Settings.default()
        super(SettingsForm, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-

import djangoplus
import os
import shutil
from fabric.api import *

password = os.environ.get('TWINE_PASSWORD')
basedir = os.path.dirname(os.path.realpath(os.path.dirname(djangoplus.__file__)))


def release(project=None):

    with lcd(basedir):
        version = None
        new_version = None
        setup_file_path = os.path.join(basedir, 'setup.py')
        setup_file_lines = open(setup_file_path).readlines()
        for i, line in enumerate(setup_file_lines):
            if 'version=' in line:
                version = line.strip()[-4:-2]
                new_version = str(int(version)+1)
                setup_file_lines[i] = setup_file_lines[i].replace(version, new_version)
                break
        if new_version:
            open(setup_file_path, 'w').write(str(''.join(setup_file_lines)))
            github_git_dir_path = os.path.join(basedir, 'djangoplus/.git')
            github_git_tmp_path = '/tmp/djangoplus-version-{}'.format(version)
            print(version, '>>>', new_version)
            print('Moving', github_git_dir_path, github_git_tmp_path)
            shutil.move(github_git_dir_path, github_git_tmp_path)
            local('python setup.py sdist')
            print('Moving', github_git_tmp_path, github_git_dir_path)
            shutil.move(github_git_tmp_path, github_git_dir_path)
            local('twine upload -p{} dist/djangoplus-0.0.{}.tar.gz'.format(password, new_version))





# -*- coding: utf-8 -*-
from djangoplus.ui import RequestComponent
from django.contrib import messages
from django.http import HttpResponseRedirect, HttpResponse


class Breadcrumbs(RequestComponent):
    """Used in the dashboard to show the sequence of pages the user has visited"""
    def __init__(self, request, view_title):
        super(Breadcrumbs, self).__init__('breadcrumbs', request)
        self.referrer = None
        if view_title:
            path = request.get_full_path()
            is_popup = 'popup=1' in path
            is_csv = 'export=csv' in path
            is_static = path.startswith('/static/')
            is_media = path.startswith('/media/')

            if not is_popup and not is_csv and not is_static and not is_media:
                if 'stack' not in request.session:
                    request.session['stack'] = []
                stack = request.session['stack']

                count = 0
                index = len(stack)
                while index:
                    index -= 1
                    title, url = stack[index]
                    if view_title == title:
                        count = len(stack) - index
                        break

                while count:
                    stack.pop()
                    count -= 1

                if stack:
                    title, url = stack[-1]
                    request.REFERRER = url
                else:
                    request.REFERRER = path

                stack.append((view_title, path))

                request.session.save()
                self.referrer = len(stack) > 1 and stack[-2][1]


def httprr(request, url, message='', error=False):
    if message:
        if error:
            messages.error(request, message, extra_tags='danger')
        else:
            messages.success(request, message, extra_tags='success')

    if 'popup' in request.GET:
        return HttpResponse(url)

    if url in ('.', '..'):
        back = abs(url == '..' and -1 or 0)
        stack = request.session.get('stack', [])
        if len(stack) >= back:
            while back:
                stack.pop()
                back -= 1
            request.session.save()
            if stack:
                title, url = stack[-1]
        else:
            url = request.get_full_path()

    if request.is_ajax():
        return HttpResponse(url)
    else:
        return HttpResponseRedirect(url)




# -*- coding: utf-8 -*-

import datetime
import calendar


def numer_of_days(start, end):
    delta = end - start
    return delta.days


def parse_date(date_string):
    if len(date_string) == 10:
        fmt = '%d/%m/%Y'
    else:
        fmt = '%d/%m/%Y %H:%M:%S'
    return datetime.datetime.strptime(date_string, fmt)


def add_months(sourcedate, months):
    month = sourcedate.month - 1 + months
    year = int(sourcedate.year + month / 12)
    month = month % 12 + 1
    day = min(sourcedate.day, calendar.monthrange(year, month)[1])
    return datetime.date(year, month, day)


def add_days(sourcedate, days):
    sourcedate = sourcedate or datetime.date.today()
    return sourcedate + datetime.timedelta(days=days)


def future(days):
    return datetime.date.today() + datetime.timedelta(days=days)


def past(days):
    return datetime.date.today() - datetime.timedelta(days=days)


def pretty_date(d):
    diff = datetime.datetime.now() - d
    s = diff.seconds
    if diff.days > 365 or diff.days < 0:
        return d.strftime('%d %b %y')
    elif 60 > diff.days > 30:
        return '1 mês atrás'
    elif diff.days > 60:
        return '{} meses atrás'.format(diff.days/30)
    elif diff.days == 1:
        return '1 dia atrás'
    elif diff.days > 1:
        return '{} dias atrás'.format(diff.days)
    elif s <= 1:
        return 'agora'
    elif s < 60:
        return '{} segundos atrás'.format(s)
    elif s < 120:
        return '1 minuto atrás'
    elif s < 3600:
        return '{} minutos atrás'.format(s/60)
    elif s < 7200:
        return '1 hora atrás'
    else:
        return '{} horas atrás'.format(s/3600)


# -*- coding: utf-8 -*-

from django.shortcuts import render
from djangoplus.utils import permissions
from djangoplus.utils.http import PdfResponse
from django.template.loader import render_to_string
from django.http.response import HttpResponseRedirect
from djangoplus.utils.metadata import iterable


def view(title, can_view=None, icon=None, menu=None, login_required=True, style='ajax', template=None, add_shortcut=False, usecase=None):

    def decorate(function):
        url = '/{}/{}/'.format(function.__module__.split('.')[-2], function.__name__)

        def receive_function_args(request, *args, **kwargs):
            without_permission = can_view and not permissions.check_group_or_permission(request, can_view)
            without_authentication = login_required and not request.user.is_authenticated
            if without_permission or without_authentication:
                    return HttpResponseRedirect('/admin/login/?next={}'.format(url))
            f_return = function(request, *args, **kwargs)
            if 'title' not in f_return:
                f_return['title'] = title
            if type(f_return) == dict:
                for key in f_return:
                    if hasattr(f_return[key], 'process_request'):
                        f_return[key].process_request()
                if 'pdf' in style:
                    request.GET._mutable = True
                    request.GET['pdf'] = 1
                    request.GET._mutable = False
                    from datetime import datetime
                    from djangoplus.admin.models import Settings
                    app_settings = Settings.default()
                    f_return['logo'] = app_settings.logo_pdf and app_settings.logo_pdf or app_settings.logo
                    f_return['project_name'] = app_settings.initials
                    f_return['project_description'] = app_settings.name
                    f_return['today'] = datetime.now()
                    template_list = ['{}.html'.format(function.__name__), 'report.html']
                    landscape = 'landscape' in style
                    return PdfResponse(render_to_string(template_list, f_return, request=request), landscape=landscape)
                else:
                    template_list = [template or '{}.html'.format(function.__name__), 'default.html']
                    return render(request, template_list, f_return)
            return f_return

        receive_function_args._view = dict(title=title, function=function, url=url, can_view=iterable(can_view), menu=menu, icon=icon,
                              style=style, add_shortcut=add_shortcut, doc=function.__doc__, usecase=usecase)
        return receive_function_args

    return decorate


def dashboard(can_view=(), position='bottom'):
    def decorate(function):
        def receive_function_args(request, *args, **kwargs):
            return function(request, *args, **kwargs)
        receive_function_args._widget = dict(function=function, can_view=can_view, position=position)
        return receive_function_args
    return decorate


def action(model, title, can_execute=(), condition=None, category='Ações',
           style='ajax', message='Ação realizada com sucesso.', template=None, inline=False, icon=None, usecase=None,
           can_execute_by_organization=None, can_execute_by_unit=None, can_execute_by_role=None, redirect_to=None):

    def decorate(function):
        def receive_function_args(request, *args, **kwargs):
            if can_execute and not permissions.check_group_or_permission(request, '{}.{}'.format(model._meta.app_label, function.__name__)):
                return HttpResponseRedirect('/admin/login/')
            f_return = function(request, *args, **kwargs)
            if 'title' not in f_return:
                f_return['title'] = title
            if type(f_return) == dict:
                if 'pdf' in style:
                    request.GET._mutable = True
                    request.GET['pdf'] = 1
                    request.GET._mutable = False
                    from datetime import datetime
                    from djangoplus.admin.models import Settings
                    app_settings = Settings.default()
                    f_return['logo'] = app_settings.logo_pdf and app_settings.logo_pdf or app_settings.logo
                    f_return['project_name'] = app_settings.initials
                    f_return['project_description'] = app_settings.name
                    f_return['today'] = datetime.now()
                    template_list = ['{}.html'.format(function.__name__), 'report.html']
                    landscape = 'landscape' in style
                    return PdfResponse(render_to_string(template_list, f_return, request=request), landscape=landscape)
                else:
                    template_list = ['{}.html'.format(function.__name__), 'default.html']
                    return render(request, template or template_list, f_return)
            else:
                return f_return

        d = dict(title=title, can_execute=iterable(can_execute), condition=condition,
                 view_name=function.__name__, function=function, group=category, css=style, message=message, model=model, input=None,
                 initial=None, choices=None, inline=inline, icon=icon, doc=function.__doc__, usecase=usecase,
                 can_execute_by_organization=iterable(can_execute_by_organization), can_execute_by_unit=iterable(can_execute_by_unit),
                 can_execute_by_role=iterable(can_execute_by_role), redirect_to=redirect_to, menu=None, display=None)

        receive_function_args._action = d

        return receive_function_args

    return decorate



# -*- coding: utf-8 -*-

from djangoplus.utils.formatter import format_value
from djangoplus.utils.metadata import get_metadata, get_fiendly_name, getattr2


def tolist(qs, add_header=True, list_display=()):
    model = qs.model
    data = []
    fields = []
    header = []
    if type(list_display) == bool or not list_display:
        list_display = get_metadata(model, 'list_display', fields)
    if list_display:
        for field_name in list_display:
            header.append(get_fiendly_name(model, field_name))
            fields.append(field_name)
    else:
        for field in get_metadata(model, 'fields'):
            header.append(field.verbose_name)
            fields.append(field.name)
    if add_header:
        data.append(header)

    for obj in qs:
        row = []
        for field in fields:
            val = getattr2(obj, field)
            if callable(val):
                val = val()
            row.append(format_value(val, False))
        data.append(row)

    return data


# -*- coding: utf-8 -*-
import os
import datetime
from selenium import webdriver
import traceback, time, json
from django.conf import settings
from djangoplus.test import cache
from django.core import serializers
from django.core.management import call_command
from selenium.webdriver.firefox.options import Options
from django.utils.translation import ugettext_lazy as _
from selenium.common.exceptions import WebDriverException
from django.contrib.staticfiles.testing import StaticLiveServerTestCase


class TestCase(StaticLiveServerTestCase):

    def __init__(self, *args, **kwargs):
        super(TestCase, self).__init__(*args, **kwargs)
        self.login_count = 0
        self.current_username = None
        self.current_password = None
        self.restored = False
        self.watched = False

    def setUp(self):
        super(TestCase, self).setUp()
        self.slowly = False
        self.username = None
        options = Options()
        options.add_argument("--window-size=1920x1080")
        if 'HEADLESS' in os.environ:
            options.add_argument("--headless")
        self.driver = webdriver.Firefox(options=options)

    def create_superuser(self, username, password):
        from djangoplus.admin.models import User
        if not User.objects.filter(username=username).exists():
            User.objects.create_superuser(username, None, password)

    def watch(self, e):
        if self.watched:
            raise e
        else:
            traceback.print_exc()
            self.watched = True
            self.driver.save_screenshot('/tmp/test.png')
            if 'HEADLESS' not in os.environ:
                input('Type enter to continue...')

    def slow_down(self):
        self.slowly = True

    def wait(self, seconds=1):
        seconds = self.slowly and 3 or seconds
        time.sleep(seconds)

    def open(self, url):
        self.driver.get("{}{}".format(self.live_server_url, url))

    def enter(self, name, value, submit=False, count=2):

        if callable(value):
            value = value()
        if type(value) == datetime.date:
            value = value.strftime('%d/%m/%Y')
        if value:
            print('{} "{}" for "{}"'.format('Entering', value, name))
        try:
            if submit:
                self.driver.execute_script("enter('{}', '{}', 1)".format(name, value))
            else:
                self.driver.execute_script("enter('{}', '{}')".format(name, value))
        except WebDriverException as e:
            if count:
                self.wait()
                self.enter(name, value, submit, count-1)
            else:
                self.watch(e)

    def choose(self, name, value, count=2):
        print('{} "{}" for "{}"'.format('Choosing', value, name))
        try:
            headless = 'HEADLESS' in os.environ and 'true' or 'false'
            self.driver.execute_script("return choose('{}', '{}', {})".format(name, value, headless))
            self.wait(2)
        except WebDriverException as e:
            if count:
                self.wait()
                self.choose(name, value, count-1)
            else:
                self.watch(e)

    def look_for(self, text, count=2):
        print('Looking for "{}"'.format(text))
        try:
            assert text in self.driver.find_element_by_tag_name('body').text
        except WebDriverException as e:
            if count:
                self.wait()
                self.look_for(text, count-1)
            else:
                self.watch(e)

    def look_at_popup_window(self, count=2):
        print('Looking at popup window')
        try:
            self.driver.execute_script("lookAtPopupWindow()")
        except WebDriverException as e:
            if count:
                self.wait()
                self.look_at_popup_window(count-1)
            else:
                self.watch(e)

    def look_at(self, text, count=2):
        print('Loking at "{}"'.format(text))
        try:
            self.driver.execute_script("lookAt('{}')".format(text))
        except WebDriverException as e:
            if count:
                self.wait()
                self.look_at(text, count-1)
            else:
                self.watch(e)

    def look_at_panel(self, text, count=2):
        print('Looking at panel "{}"'.format(text))
        try:
            self.driver.execute_script("lookAtPanel('{}')".format(text))
        except WebDriverException as e:
            if count:
                self.wait()
                self.look_at_panel(text, count-1)
            else:
                self.watch(e)

    def click_menu(self, *texts):
        print('Clicking menu "{}"'.format('->'.join(texts)))
        self.wait()
        for text in texts:
            self.wait()
            try:
                self.driver.execute_script("clickMenu('{}')".format(text.strip()))
            except WebDriverException as e:
                self.watch(e)
        self.wait()

    def click_link(self, text):
        print('Clicking link "{}"'.format(text))
        try:
            self.driver.execute_script("clickLink('{}')".format(text))
        except WebDriverException as e:
            self.watch(e)
        self.wait()

    def click_button(self, text):
        print('Clicking button "{}"'.format(text))
        try:
            self.driver.execute_script("clickButton('{}')".format(text))
        except WebDriverException as e:
            self.watch(e)
        self.wait()

    def click_tab(self, text):
        print('Clicking tab "{}"'.format(text))
        try:
            self.driver.execute_script("clickTab('{}')".format(text))
        except WebDriverException as e:
            self.watch(e)
        self.wait()

    def click_icon(self, name):
        print('Clicking icon "{}"'.format(name))
        try:
            self.driver.execute_script("clickIcon('{}')".format(name))
        except WebDriverException as e:
            self.watch(e)
        self.wait()

    def login(self, username, password):
        print('Logging as', username)
        self.login_count += 1
        self.current_username = username
        self.current_password = password

        if self.login_count >= cache.LOGIN_COUNT:

            self.dump()

            self.open('/admin/login/')
            self.enter(_('Username'), username)
            self.enter(_('Password'), password, True)
            self.wait()

            return True

        else:
            if not self.restored:
                self.restore()
            return False

    def logout(self):
        print('Logging out')
        self.click_icon('Configurações')
        self.wait()
        self.click_link('Sair')
        self.wait()
        self.username = None

    def tearDown(self):
        super(TestCase, self).tearDown()
        self.driver.close()
        self.driver.service.stop()
        # len(self._resultForDoCleanups.errors)>0

    def dump(self, failed=False):
        file_path = '/tmp/{}.test'.format(settings.PROJECT_NAME)
        dump_file_path = '/tmp/{}.json'.format(settings.PROJECT_NAME)

        data = dict(login_count=self.login_count, username=self.current_username, password=self.current_password)
        open(file_path, 'w').write(json.dumps(data))
        output = open(dump_file_path,'w')
        app_labels = []
        for app in settings.INSTALLED_APPS:
            app_label = app.split('.')[-1]
            if app_label not in 'auth':
                app_labels.append(app_label)
        call_command('dumpdata', *app_labels, format='json', indent=3, stdout=output)
        output.close()

    def restore(self):
        dump_file_path = '/tmp/{}.json'.format(settings.PROJECT_NAME)
        call_command('loaddata', dump_file_path)
        self.restored = True

    def pause(self):
        import pdb
        pdb.set_trace()



# Generated by Django 2.0.3 on 2018-04-16 10:34

from django.db import migrations


def migrate(apps, schema_editor):
    Organization = apps.get_model('admin', 'Organization')
    Unit = apps.get_model('admin', 'Unit')
    Scope = apps.get_model('admin', 'Scope')
    for i, scope in enumerate([Organization, Unit]):
        for cls in scope.__subclasses__():
            for obj in cls.objects.all():
                pk = obj.pk + (1000 * i)
                Scope.objects.get_or_create(pk=pk)
                scope_lookup = {'id': obj.pk}
                scope_update = {'id': pk}
                scope.objects.filter(**scope_lookup).update(**scope_update)
                cls_lookup = {'{}_ptr_id'.format(scope.__name__.lower()): obj.pk}
                cls_update = {'{}_ptr_id'.format(scope.__name__.lower()): pk}
                cls.objects.filter(**cls_lookup).update(**cls_update)
                for rel in cls._meta.related_objects:
                    if rel.remote_field.model._meta.app_label == 'admin':
                        continue
                    lookup = {rel.remote_field.name: obj.pk}
                    update = {rel.remote_field.name: pk}
                    rel.remote_field.model.objects.filter(**lookup).update(**update)


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0002_scope'),
    ]

    operations = [migrations.RunPython(migrate)]
